==> comphep-4.4.110.tgz: comphep-4.4.110/src/symb/sos.c <==
/*
* Copyright (C) 2001-2008, CompHEP Collaboration
* Copyright (C) 1997, Alexander Pukhov
* ------------------------------------------------------
* $Id: sos.c,v 1.10 2003/11/08 19:46:05 kryukov Exp $
*
* $Log: sos.c,v $
* Revision 1.10  2003/11/08 19:46:05  kryukov
* To bring into according the usage of messanykey and warnanykey functions.
*
* Revision 1.9  2003/10/29 16:24:49  sherstnv
* Bug fixed: "nin" and "nout" parameters is not stored/read to/from the tmp/safe file
*
* Revision 1.8  2003/09/22 11:34:09  sherstnv
* Process entering divides to 2 stages:
* 1) At first, we enter beams (name, mass, content, str,fun)
* 2) after that - final state.
*
* Revision 1.7  2003/06/25 18:46:42  kryukov
* *** empty log message ***
*
* Revision 1.6  2003/06/15 12:13:31  sherstnv
* New schime of excluding/keeping diagrams during their construction
* is realized. Extra conditions are included:
* ! - "not equaled"
* = - "equaled"
* < - "less than"
*
* All necessary help files is corrected.
*
* Revision 1.5  2003/04/22 09:01:20  kryukov
* Apply indent to improve readability of files
*
* Revision 1.4  2003/04/21 20:09:48  kryukov
* Misprints and English improve. Special thanks M.D.
*
* Revision 1.3  2003/04/20 17:55:02  kryukov
* Clean up warnings during make
*/
#include <string.h>
#include <math.h>

#include "service2/include/chep_limits.h"
#include "service2/include/getmem.h"
#include "service2/include/unix_utils.h"
#include "service2/include/syst.h"
#include "service2/include/s_files.h"
#include "chep_crt/include/chep_crt.h"

#include "physics.h"
#include "sos.h"
#include "c_out.h"
#include "prepdiag.h"
#include "process.h"
#include "process_core.h"
#include "model.h"
#include "cweight.h"


static void
init_safe (int *exitlevel)
{
  int i;

  strcpy (beam[0].h.name, "e");
  strcpy (beam[1].h.name, "E");
  beam[0].energy = 500;
  beam[1].energy = 500;
  
  setFinalstatech ("m,M");
  
  for (i = 0; i < MAXINOUT; i++)
    {
      strcpy (hadrons[i].name, "");
      strcpy (hadrons[i].strfun, "");
      hadrons[i].how = 0;
    }
  setExclprtlist ("");
  setKeepprtlist ("");
  setModelNumberSymb (4);
  *exitlevel = 0;
}


void
restoreent (int *exitlevel)
{
  int i;
  int num;
  int d[32];
  shortstr tmps;
  double e[2], p[2];
  FILE *ff = fopen (scat ("%stmp%csafe", pathtouser, f_slash), "r");

  if (ff != NULL) {
      d[0]  = fscanf (ff, "#Model %d\n",             &num);
      setModelNumberSymb (num);
      d[1]  = fscanf (ff, "#nIn %d\n",               &num);
      setnin (num);
      d[2]  = fscanf (ff, "#nOut %d\n",              &num);
      setnout (num);
      d[3]  = fscanf (ff, "#beam_1 %[^\n]\n",         beam[0].h.name);
      d[4]  = fscanf (ff, "#beam_2 %[^\n]\n",         beam[1].h.name);
      d[5]  = fscanf (ff, "#beam_energy_1 %lf\n",   &(beam[0].energy));
      d[6]  = fscanf (ff, "#beam_energy_2 %lf\n",   &(beam[1].energy));
      for (i = 0; i < 2; ++i) {
        double m = beam[i].h.mass;
        e[i] = beam[i].energy;
        p[i] = sqrt (e[i] * e[i] - m * m);
      }
      set_sqrts_rap (e[0],p[0],e[1],p[1]);
      d[7]  = fscanf (ff, "#strfun_1 %[^\n]\n",       beam[0].h.strfun);
      d[8]  = fscanf (ff, "#strfun_2 %[^\n]\n",       beam[1].h.strfun);
      d[9]  = fscanf (ff, "#Final_state %[^\n]\n",    tmps);
      trim (tmps);
      setFinalstatech (tmps);
      d[10]  = fscanf (ff, "#Remove_Virtual%[^\n]\n", tmps);
      trim (tmps);
      setExclprtlist (tmps);
      d[11]  = fscanf (ff, "#Keep_Virtual%[^\n]\n",   tmps);
      trim (tmps);
      setKeepprtlist (tmps);
      d[12] = fscanf (ff, "#nSubproc(ampl) %d\n",    &subproc_f);
      d[13] = fscanf (ff, "#nSubproc(squared) %d\n", &subproc_sq);
      d[14] = fscanf (ff, "#ConservationLaw %d\n",   &consLow);
      d[15] = fscanf (ff, "#Nc==inf %d\n",           &num);
      setNcinflimit (num);
      d[16] = fscanf (ff, "#ExitCode %d\n",           exitlevel);
 
      if (1 != d[0] || 1 != d[1] || 1 != d[2] || 1 != d[3] || 1 != d[4] ||
          1 != d[5] || 1 != d[6] || 1 != d[7] || 1 != d[8] || 1 != d[9] ||
          1 != d[10] || 1 != d[11] || 1 != d[12] || 1 != d[13] || 1 != d[14] ||
          1 != d[14] || 1 != d[16]
          )
          init_safe (exitlevel);

      fclose (ff);
      if (1 == getnin ())
        {
          shortstr tmps;
	  sprintf (tmps, "%s -> %s", beam[0].h.name, getFinalstatech ());
	  setProcessch (tmps);
        }
      else
        {
          shortstr tmps;
          sprintf (tmps, "%s,%s -> %s", beam[0].h.name, beam[1].h.name, getFinalstatech ());
	  setProcessch (tmps);
        }
    }
  else
    init_safe (exitlevel);
}


int 
restoreent_dump (void)
{
  int subproc = 0;
  int num;
  double num1;
  int d[32];
  shortstr tmps;
  FILE *ff = fopen (scat ("%stmp%csafe", pathtouser, f_slash), "r");

  if (ff != NULL) {
    d[0]  = fscanf (ff, "#Model %d\n",             &num);
    d[1]  = fscanf (ff, "#nIn %d\n",               &num);
    d[2]  = fscanf (ff, "#nOut %d\n",              &num);
    d[3]  = fscanf (ff, "#beam_1 %[^\n]\n",         tmps);
    d[4]  = fscanf (ff, "#beam_2 %[^\n]\n",         tmps);
    d[5]  = fscanf (ff, "#beam_energy_1 %lf\n",    &num1);
    d[6]  = fscanf (ff, "#beam_energy_2 %lf\n",    &num1);
    d[7]  = fscanf (ff, "#strfun_1 %[^\n]\n",       tmps);
    d[8]  = fscanf (ff, "#strfun_2 %[^\n]\n",       tmps);
    d[9]  = fscanf (ff, "#Final_state %[^\n]\n",    tmps);
    d[10] = fscanf (ff, "#Remove_Virtual%[^\n]\n",  tmps);
    d[11] = fscanf (ff, "#Keep_Virtual%[^\n]\n",    tmps);
    d[12] = fscanf (ff, "#nSubproc(ampl) %d\n",    &num);
    d[13] = fscanf (ff, "#nSubproc(squared) %d\n", &subproc);
    d[14] = fscanf (ff, "#ConservationLaw %d\n",   &num);
    d[15] = fscanf (ff, "#Nc==inf %d\n",           &num);
    d[16] = fscanf (ff, "#ExitCode %d\n",          &num);
  }
  return subproc;
}


void
saveent (int exitlevel)
{
  FILE *ff = fopen (scat ("%stmp%csafe", pathtouser, f_slash), "w");

  fprintf (ff, "#Model %i\n",             getModelNumberSymb());
  fprintf (ff, "#nIn %i\n",               getnin());
  fprintf (ff, "#nOut %i\n",              getnout());
  fprintf (ff, "#beam_1 %s\n",            beam[0].h.name);
  fprintf (ff, "#beam_2 %s\n",            beam[1].h.name);
  fprintf (ff, "#beam_energy_1 %f\n",     beam[0].energy);
  fprintf (ff, "#beam_energy_2 %f\n",     beam[1].energy);
  fprintf (ff, "#strfun_1 %s\n",          beam[0].h.strfun);
  fprintf (ff, "#strfun_2 %s\n",          beam[1].h.strfun);
  fprintf (ff, "#Final_state %s\n",       getFinalstatech ());
  fprintf (ff, "#Remove_Virtual %s\n",    getExclprtlist ());
  fprintf (ff, "#Keep_Virtual %s\n",      getKeepprtlist ());
  fprintf (ff, "#nSubproc(ampl) %d\n",    subproc_f);
  fprintf (ff, "#nSubproc(squared) %d\n", subproc_sq);
  fprintf (ff, "#ConservationLaw %d\n",   consLow);
  fprintf (ff, "#Nc==inf %d\n",           getNcinflimit ());
  fprintf (ff, "#ExitCode %d\n",          exitlevel);
  fclose (ff);
}

void
prepare_process_dat (void)
{
  FILE *ff = fopen (scat ("%sprocess_gen.dat", pathtouser, f_slash), "w");

  fprintf (ff, "############################################################\n");
  fprintf (ff, "#         Data file for symb_script.pl\n");
  fprintf (ff, "#       For the symbolic batch script version 1.0\n");
  fprintf (ff, "############################################################\n\n");
  fprintf (ff, "#  You have to set the model number, which you are going to use.\n");
  fprintf (ff, "#  The model number corresponds to the string number of the model\n");
  fprintf (ff, "#  in the CompHEP model menu in the GUI mode. \n");
  fprintf (ff, "model number: %i\n\n", getModelNumberSymb());
  fprintf (ff, "#  Beam names can be taken from a table of beams \n");
  fprintf (ff, "#  (see CompHEP in the GUI regime). Energy unit is GeV\n");
  fprintf (ff, "beam 1: %s\n", beam[0].h.name);
  fprintf (ff, "beam 2: %s\n", beam[1].h.name);
  fprintf (ff, "beam energy 1: %f\n",   beam[0].energy);
  fprintf (ff, "beam energy 2: %f\n\n", beam[1].energy);
  fprintf (ff, "#  This string defines the final state of your process. Model \n");
  fprintf (ff, "#  particles and composite particles (see the corresponding table)\n");
  fprintf (ff, "#  can be used. \n");
  fprintf (ff, "final state:%s\n\n", getFinalstatech ());
  fprintf (ff, "#  If you'd like to exclude feynman diagrams with some model\n");
  fprintf (ff, "#  particles (in propagators!), enter the particles here\n");
  fprintf (ff, "exclude diagrams with: %s\n", getExclprtlist ());
  fprintf (ff, "\n");
  fprintf (ff, "#  If you'd like to keep feynman diagrams with some model\n");
  fprintf (ff, "#  particles (in propagators!), enter the particles here\n");
  fprintf (ff, "keep diagrams with: %s\n\n", getKeepprtlist ());
  fprintf (ff, "#  If you enter no, s_comphep generates diagrams and does not\n");
  fprintf (ff, "#  do symbolic calculations.\n");
  fprintf (ff, "make symbolic calculations(yes/no): yes\n\n");
  fprintf (ff, "#  If you enter no, comphep calculates all squared diagrams, \n");
  fprintf (ff, "#  but n_comphep will not be created.\n");
  fprintf (ff, "make n_comphep generator(yes/no): yes\n");
  fclose (ff);
}

void
save_sos (int ercode)
{
  unsigned nproc;
  csdiagram cd;
  marktp mark;

  if (ercode == -2)             /* User Break */
    {
      saveent (7);
      finish ("Restart of the CompHEP program.");
      exit (13);                /*  Restart  */
    }

  if (ercode == -1)             /* Heap is empty */
    {
      mark.blk_ = NULL;
      mark.pos_ = 0;
      release_ (&mark);
    }                           /* Heap is empty, continue */

  /*  TooLargeNumber  */
  /*  TooManyIdentifiers  */
  /*  RangeCheckError  */
  if ((ercode < 0) || (ercode == 7) || (ercode == 11) || (ercode == 12))        /*  Restart  */
    {
      saveent (9);
      nproc = ftell (diagrq) - sizeof (cd);
      fseek (diagrq, nproc, SEEK_SET);
      FREAD1 (cd, diagrq);
      cd.status = -2;
      fseek (diagrq, nproc, SEEK_SET);
      FWRITE1 (cd, diagrq);
      finish ("*** Error! CompHEP is restarted");
      exit (13);                /*  Restart  */
    }

/*  not disk space  */
  if ((ercode == 40))
    {
      /*  freez */
      saveent (6);
      finish ("*** Error!");
      exit (0);                 /*  End of work  */
    }
  if (ercode == 14)
    {
      saveent (1);
      warnanykey (10, 10, " Check model !");
      finish ("*** Error!");
      exit (13);
    }
}


int
batch_composite (char *name, shortstr h)
{
  int stop = 0;
  char nm[100], buff[500];
  /*char content[500]; */
  FILE *f = NULL;

  f = fopen (scat ("%stmp%ccomposite.dat", pathtouser, f_slash), "r");
  if (f != NULL)
    {
      while (!stop)
        {
          if (fscanf (f, "%[^\n]", buff))
            {
              int j = 0, i = -1;
              while (buff[i++] != ':')
                nm[i] = buff[i];
              nm[i - 1] = '\0';
              if (!strcmp (name, nm))
                {
                  while (buff[i + j] != '\0')
                    {
                      nm[j] = buff[i + j];
                      j++;
                    }
                  strncpy (h, nm, j);
                  fclose (f);
                  return 0;
                }
            }
          else
            {
              sprintf (buff, "composite.dat does non contain the particle %s", name);
              fclose (f);
              batch_error (buff, 1);
              stop = 1;
            }
        }
    }
  batch_error ("File composite.dat have to be located in tmp", 1);
  return 1;
}

==> comphep-4.4.113.tgz: comphep-4.4.113/src/symb/sos.c <==
/*
* Copyright (C) 2001-2008, CompHEP Collaboration
* Copyright (C) 1997, Alexander Pukhov
* ------------------------------------------------------
* $Id: sos.c,v 1.10 2003/11/08 19:46:05 kryukov Exp $
*
* $Log: sos.c,v $
* Revision 1.10  2003/11/08 19:46:05  kryukov
* To bring into according the usage of messanykey and warnanykey functions.
*
* Revision 1.9  2003/10/29 16:24:49  sherstnv
* Bug fixed: "nin" and "nout" parameters is not stored/read to/from the tmp/safe file
*
* Revision 1.8  2003/09/22 11:34:09  sherstnv
* Process entering divides to 2 stages:
* 1) At first, we enter beams (name, mass, content, str,fun)
* 2) after that - final state.
*
* Revision 1.7  2003/06/25 18:46:42  kryukov
* *** empty log message ***
*
* Revision 1.6  2003/06/15 12:13:31  sherstnv
* New schime of excluding/keeping diagrams during their construction
* is realized. Extra conditions are included:
* ! - "not equaled"
* = - "equaled"
* < - "less than"
*
* All necessary help files is corrected.
*
* Revision 1.5  2003/04/22 09:01:20  kryukov
* Apply indent to improve readability of files
*
* Revision 1.4  2003/04/21 20:09:48  kryukov
* Misprints and English improve. Special thanks M.D.
*
* Revision 1.3  2003/04/20 17:55:02  kryukov
* Clean up warnings during make
*/
#include <string.h>
#include <math.h>

#include "service2/include/chep_limits.h"
#include "service2/include/getmem.h"
#include "service2/include/unix_utils.h"
#include "service2/include/syst.h"
#include "service2/include/s_files.h"
#include "chep_crt/include/chep_crt.h"

#include "physics.h"
#include "sos.h"
#include "prepdiag.h"
#include "process.h"
#include "process_core.h"
#include "model.h"
#include "cweight.h"


static void
init_safe (int *exitlevel)
{
  int i;

  strcpy (beam[0].h.name, "p");
  strcpy (beam[1].h.name, "p");
  beam[0].energy = 7000.0;
  beam[1].energy = 7000.0;
  
  setFinalstatech ("m,M,G");
  
  for (i = 0; i < MAXINOUT; i++)
    {
      strcpy (hadrons[i].name, "");
      strcpy (hadrons[i].strfun, "");
      hadrons[i].how = 0;
    }
  setExclprtlist ("");
  setKeepprtlist ("");
  setModelNumberSymb (4);
  *exitlevel = 0;
}


void
restoreent (int *exitlevel)
{
  int i;
  int num;
  int d[32];
  shortstr tmps;
  double e[2], p[2];
  FILE *ff = fopen (scat ("%stmp%csafe", pathtouser, f_slash), "r");

  if (ff != NULL) {
      d[0]  = fscanf (ff, "#Model %d\n",             &num);
      setModelNumberSymb (num);
      d[1]  = fscanf (ff, "#nIn %d\n",               &num);
      setnin (num);
      d[2]  = fscanf (ff, "#nOut %d\n",              &num);
      setnout (num);
      d[3]  = fscanf (ff, "#beam_1 %[^\n]\n",         beam[0].h.name);
      d[4]  = fscanf (ff, "#beam_2 %[^\n]\n",         beam[1].h.name);
      d[5]  = fscanf (ff, "#beam_energy_1 %lf\n",   &(beam[0].energy));
      d[6]  = fscanf (ff, "#beam_energy_2 %lf\n",   &(beam[1].energy));
      for (i = 0; i < 2; ++i) {
        double m = beam[i].h.mass;
        e[i] = beam[i].energy;
        p[i] = sqrt (e[i] * e[i] - m * m);
      }
      set_sqrts_rap (e[0],p[0],e[1],p[1]);
      d[7]  = fscanf (ff, "#strfun_1 %[^\n]\n",       beam[0].h.strfun);
      d[8]  = fscanf (ff, "#strfun_2 %[^\n]\n",       beam[1].h.strfun);
      d[9]  = fscanf (ff, "#Final_state %[^\n]\n",    tmps);
      trim (tmps);
      setFinalstatech (tmps);
      d[10]  = fscanf (ff, "#Remove_Virtual%[^\n]\n", tmps);
      trim (tmps);
      setExclprtlist (tmps);
      d[11]  = fscanf (ff, "#Keep_Virtual%[^\n]\n",   tmps);
      trim (tmps);
      setKeepprtlist (tmps);
      d[12] = fscanf (ff, "#nSubproc(ampl) %d\n",    &subproc_f);
      d[13] = fscanf (ff, "#nSubproc(squared) %d\n", &subproc_sq);
      d[14] = fscanf (ff, "#ConservationLaw %d\n",   &consLow);
      d[15] = fscanf (ff, "#Nc==inf %d\n",           &num);
      setNcinflimit (num);
      d[16] = fscanf (ff, "#ExitCode %d\n",           exitlevel);
 
      if (1 != d[0] || 1 != d[1] || 1 != d[2] || 1 != d[3] || 1 != d[4] ||
          1 != d[5] || 1 != d[6] || 1 != d[7] || 1 != d[8] || 1 != d[9] ||
          1 != d[10] || 1 != d[11] || 1 != d[12] || 1 != d[13] || 1 != d[14] ||
          1 != d[14] || 1 != d[16]
          )
          init_safe (exitlevel);

      fclose (ff);
      if (1 == getnin ())
        {
          shortstr tmps;
	  sprintf (tmps, "%s -> %s", beam[0].h.name, getFinalstatech ());
	  setProcessch (tmps);
        }
      else
        {
          shortstr tmps;
          sprintf (tmps, "%s,%s -> %s", beam[0].h.name, beam[1].h.name, getFinalstatech ());
	  setProcessch (tmps);
        }
    }
  else
    init_safe (exitlevel);
}


int 
restoreent_dump (void)
{
  int subproc = 0;
  int num;
  double num1;
  int d[32];
  shortstr tmps;
  FILE *ff = fopen (scat ("%stmp%csafe", pathtouser, f_slash), "r");

  if (ff != NULL) {
    d[0]  = fscanf (ff, "#Model %d\n",             &num);
    d[1]  = fscanf (ff, "#nIn %d\n",               &num);
    d[2]  = fscanf (ff, "#nOut %d\n",              &num);
    d[3]  = fscanf (ff, "#beam_1 %[^\n]\n",         tmps);
    d[4]  = fscanf (ff, "#beam_2 %[^\n]\n",         tmps);
    d[5]  = fscanf (ff, "#beam_energy_1 %lf\n",    &num1);
    d[6]  = fscanf (ff, "#beam_energy_2 %lf\n",    &num1);
    d[7]  = fscanf (ff, "#strfun_1 %[^\n]\n",       tmps);
    d[8]  = fscanf (ff, "#strfun_2 %[^\n]\n",       tmps);
    d[9]  = fscanf (ff, "#Final_state %[^\n]\n",    tmps);
    d[10] = fscanf (ff, "#Remove_Virtual%[^\n]\n",  tmps);
    d[11] = fscanf (ff, "#Keep_Virtual%[^\n]\n",    tmps);
    d[12] = fscanf (ff, "#nSubproc(ampl) %d\n",    &num);
    d[13] = fscanf (ff, "#nSubproc(squared) %d\n", &subproc);
    d[14] = fscanf (ff, "#ConservationLaw %d\n",   &num);
    d[15] = fscanf (ff, "#Nc==inf %d\n",           &num);
    d[16] = fscanf (ff, "#ExitCode %d\n",          &num);
  }
  return subproc;
}


void
saveent (int exitlevel)
{
  FILE *ff = fopen (scat ("%stmp%csafe", pathtouser, f_slash), "w");

  fprintf (ff, "#Model %i\n",             getModelNumberSymb());
  fprintf (ff, "#nIn %i\n",               getnin());
  fprintf (ff, "#nOut %i\n",              getnout());
  fprintf (ff, "#beam_1 %s\n",            beam[0].h.name);
  fprintf (ff, "#beam_2 %s\n",            beam[1].h.name);
  fprintf (ff, "#beam_energy_1 %f\n",     beam[0].energy);
  fprintf (ff, "#beam_energy_2 %f\n",     beam[1].energy);
  fprintf (ff, "#strfun_1 %s\n",          beam[0].h.strfun);
  fprintf (ff, "#strfun_2 %s\n",          beam[1].h.strfun);
  fprintf (ff, "#Final_state %s\n",       getFinalstatech ());
  fprintf (ff, "#Remove_Virtual %s\n",    getExclprtlist ());
  fprintf (ff, "#Keep_Virtual %s\n",      getKeepprtlist ());
  fprintf (ff, "#nSubproc(ampl) %d\n",    subproc_f);
  fprintf (ff, "#nSubproc(squared) %d\n", subproc_sq);
  fprintf (ff, "#ConservationLaw %d\n",   consLow);
  fprintf (ff, "#Nc==inf %d\n",           getNcinflimit ());
  fprintf (ff, "#ExitCode %d\n",          exitlevel);
  fclose (ff);
}

void
save_sos (int ercode)
{
  unsigned nproc;
  csdiagram cd;
  marktp mark;

  if (ercode == -2)             /* User Break */
    {
      saveent (7);
      finish ("Restart of the CompHEP program.");
      exit (13);                /*  Restart  */
    }

  if (ercode == -1)             /* Heap is empty */
    {
      mark.blk_ = NULL;
      mark.pos_ = 0;
      release_ (&mark);
    }                           /* Heap is empty, continue */

  /*  TooLargeNumber  */
  /*  TooManyIdentifiers  */
  /*  RangeCheckError  */
  if ((ercode < 0) || (ercode == 7) || (ercode == 11) || (ercode == 12))        /*  Restart  */
    {
      saveent (9);
      nproc = ftell (diagrq) - sizeof (cd);
      fseek (diagrq, nproc, SEEK_SET);
      FREAD1 (cd, diagrq);
      cd.status = -2;
      fseek (diagrq, nproc, SEEK_SET);
      FWRITE1 (cd, diagrq);
      finish ("*** Error! CompHEP is restarted");
      exit (13);                /*  Restart  */
    }

/*  not disk space  */
  if ((ercode == 40))
    {
      /*  freez */
      saveent (6);
      finish ("*** Error!");
      exit (0);                 /*  End of work  */
    }
  if (ercode == 14)
    {
      saveent (1);
      warnanykey (10, 10, " Check model !");
      finish ("*** Error!");
      exit (13);
    }
}


int
batch_composite (char *name, shortstr h)
{
  int stop = 0;
  char nm[100], buff[500];
  /*char content[500]; */
  FILE *f = NULL;

  f = fopen (scat ("%stmp%ccomposite.dat", pathtouser, f_slash), "r");
  if (f != NULL)
    {
      while (!stop)
        {
          if (fscanf (f, "%[^\n]", buff))
            {
              int j = 0, i = -1;
              while (buff[i++] != ':')
                nm[i] = buff[i];
              nm[i - 1] = '\0';
              if (!strcmp (name, nm))
                {
                  while (buff[i + j] != '\0')
                    {
                      nm[j] = buff[i + j];
                      j++;
                    }
                  strncpy (h, nm, j);
                  fclose (f);
                  return 0;
                }
            }
          else
            {
              sprintf (buff, "composite.dat does non contain the particle %s", name);
              fclose (f);
              batch_error (buff, 1);
              stop = 1;
            }
        }
    }
  batch_error ("File composite.dat have to be located in tmp", 1);
  return 1;
}
